==> sqlentity-2.1.1.tar.gz: lib/dao/dao.dart <==
import 'dart:async';

import 'package:sqflite/sqflite.dart';
import 'package:sqlentity/base-entity/entity.dart';
import 'package:sqlentity/dao/i-dao.dart';
import 'package:sqlentity/database/abstract-database.dart';

///Repository
class DAO<T extends Entity> extends AbstractRepository implements IDAO<T> {
  T _entity;

  ///init repository
  DAO(T entity) : super() {
    this._entity = entity;
  }

  ///table
  T get entity => _entity;

  ///insert data table
  @override
  Future<int> insert(T entity) async {
    var results;
    try {
      var sql = 'INSERT INTO ${entity.table}(';
      entity.column.forEach((values) {
        if (values.column != entity.column[0].column) {
          if (values == entity.column[entity.column.length - 1])
            sql += "${values.column} ";
          else
            sql += "${values.column} ,";
        }
      });
      sql += ") VALUES (";
      entity.column.forEach((column) {
        if (column.column != entity.column[0].column) if (column ==
            entity.column[entity.column.length - 1])
          sql += "? ";
        else
          sql += "? ,";
      });
      sql += ")";

      await open();
      if (!(database != null && database.isOpen)) return -1;
      int value;
      await database.transaction((txn) async {
        try {
          var batch = await txn.batch();
          await batch.rawInsert(sql, entity.getValues());
          results = await batch.commit();
        } on DatabaseException catch (e) {
          if (e.toString().contains("UNIQUE")) {
            value = 0;
          }
        }
      });
      await close();
      if (value == 0) {
        return await 0;
      }
      return await results[0];
    } catch (e) {
      await close();
      return await -1;
    }
  }

  ///search data table
  @override
  Future<List<T>> select() async {
    try {
      await open();
      if (!(database != null && database.isOpen)) return null;
      var select = await database.rawQuery('SELECT * FROM ${_entity.table}');
      await close();

      if (select == null) {
        return null;
      }

      return await select.map((i) => _entity.map(i)).toList();
    } catch (e) {
      await close();
      return null;
    }
  }

  ///update data table
  @override
  Future<bool> update(T entity) async {
    try {
      var sql = 'UPDATE ${entity.table} SET ';
      entity.column.forEach((values) {
        if (values.column != entity.column[0].column) {
          if (values == entity.column[entity.column.length - 1])
            sql += "${values.column} = ? ";
          else
            sql += "${values.column} = ? ,";
        }
      });
      sql += " Where ${entity.column[0].column} = ${entity.column[0].value}";

      await open();
      if (!(database != null && database.isOpen)) return false;
      int results = 1;
      await database.transaction((txn) async {
        try {
          var batch = txn.batch();
          await batch.rawUpdate(sql, entity.getValues());
          await batch.commit();
        } on DatabaseException catch (e) {
          if (e.toString().contains("UNIQUE")) {
            results = 0;
          }
        }
      });
      await close();
      if (results == 0) {
        return await true;
      }
      return await true;
    } catch (e) {
      await close();
      return await false;
    }
  }

  ///delete data table
  @override
  Future<bool> delete(T entity) async {
    try {
      var sql =
          'DELETE FROM  ${entity.table} WHERE ${entity.column[0].column} = ?';

      await open();
      if (!(database != null && database.isOpen)) return null;
      await database.transaction((txn) async {
        var batch = txn.batch();
        await batch.rawDelete(sql, [entity.column[0].value]);
        await batch.commit();
      });

      await close();
      return await true;
    } catch (e) {
      await close();
      return await false;
    }
  }

  ///search for id data table
  @override
  Future<T> getById(T entity) async {
    try {
      await open();
      if (!(database != null && database.isOpen)) return null;

      var select = await database.rawQuery(
          'SELECT * FROM ${_entity.table} WHERE ${entity.column[0].column} = ?',
          [entity.column[0].value]);
      await close();
      return await _entity.map(select[0]) as T;
    } catch (e) {
      await close();
      return null;
    }
  }

  ///count data table
  @override
  Future<int> count() async {
    try {
      await open();
      if (!(database != null && database.isOpen)) return null;
      var count = await Sqflite.firstIntValue(
          await database.rawQuery("SELECT COUNT(*) FROM ${_entity.table}"));
      await close();

      if (count == null) {
        return 0;
      }

      return await count;
    } catch (e) {
      await close();
      return -1;
    }
  }
}

==> sqlentity-2.1.3.tar.gz: lib/dao/dao.dart <==
import 'dart:async';

import 'package:sqflite/sqflite.dart';
import 'package:sqlentity/base-entity/entity.dart';
import 'package:sqlentity/dao/i-dao.dart';
import 'package:sqlentity/database/abstract-database.dart';

///Repository
class DAO<T extends Entity> extends AbstractRepository implements IDAO<T> {
  T _entity;

  ///init repository
  DAO(T entity) : super() {
    this._entity = entity;
  }

  ///table
  T get entity => _entity;

  ///insert data table
  @override
  Future<int> insert(T entity) async {
    var results;
    try {
      var sql = 'INSERT INTO ${entity.table}(';
      entity.column.forEach((values) {
        if (values.column != entity.column[0].column) {
          if (values == entity.column[entity.column.length - 1])
            sql += "${values.column} ";
          else
            sql += "${values.column} ,";
        }
      });
      sql += ") VALUES (";
      entity.column.forEach((column) {
        if (column.column != entity.column[0].column) if (column ==
            entity.column[entity.column.length - 1])
          sql += "? ";
        else
          sql += "? ,";
      });
      sql += ")";

      await open();
      if (!(database != null && database.isOpen)) return -1;
      int value;
      await database.transaction((txn) async {
        try {
          var batch = txn.batch();
          batch.rawInsert(sql, entity.getValues());
          results = await batch.commit();
        } on DatabaseException catch (e) {
          if (e.toString().contains("UNIQUE")) {
            value = 0;
          }
        }
      });
      await close();
      if (value == 0) {
        return 0;
      }
      return await results[0];
    } catch (e) {
      await close();
      return -1;
    }
  }

  ///search data table
  @override
  Future<List<T>> select() async {
    try {
      await open();
      if (!(database != null && database.isOpen)) return null;
      var select = await database.rawQuery('SELECT * FROM ${_entity.table}');
      await close();

      if (select == null) {
        return null;
      }

      return select.map((i) => _entity.map(i)).toList();
    } catch (e) {
      await close();
      return null;
    }
  }

  ///update data table
  @override
  Future<bool> update(T entity) async {
    try {
      var sql = 'UPDATE ${entity.table} SET ';
      entity.column.forEach((values) {
        if (values.column != entity.column[0].column) {
          if (values == entity.column[entity.column.length - 1])
            sql += "${values.column} = ? ";
          else
            sql += "${values.column} = ? ,";
        }
      });
      sql += " Where ${entity.column[0].column} = ${entity.column[0].value}";

      await open();
      if (!(database != null && database.isOpen)) return false;
      int results = 1;
      await database.transaction((txn) async {
        try {
          var batch = txn.batch();
          batch.rawUpdate(sql, entity.getValues());
          await batch.commit();
        } on DatabaseException catch (e) {
          if (e.toString().contains("UNIQUE")) {
            results = 0;
          }
        }
      });
      await close();
      if (results == 0) {
        return true;
      }
      return true;
    } catch (e) {
      await close();
      return false;
    }
  }

  ///delete data table
  @override
  Future<bool> delete(T entity) async {
    try {
      var sql =
          'DELETE FROM  ${entity.table} WHERE ${entity.column[0].column} = ?';

      await open();
      if (!(database != null && database.isOpen)) return null;
      await database.transaction((txn) async {
        var batch = txn.batch();
        batch.rawDelete(sql, [entity.column[0].value]);
        await batch.commit();
      });

      await close();
      return true;
    } catch (e) {
      await close();
      return false;
    }
  }

  ///search for id data table
  @override
  Future<T> getById(T entity) async {
    try {
      await open();
      if (!(database != null && database.isOpen)) return null;

      var select = await database.rawQuery(
          'SELECT * FROM ${_entity.table} WHERE ${entity.column[0].column} = ?',
          [entity.column[0].value]);
      await close();
      return _entity.map(select[0]) as T;
    } catch (e) {
      await close();
      return null;
    }
  }

  ///count data table
  @override
  Future<int> count() async {
    try {
      await open();
      if (!(database != null && database.isOpen)) return null;
      var count = Sqflite.firstIntValue(
          await database.rawQuery("SELECT COUNT(*) FROM ${_entity.table}"));
      await close();

      if (count == null) {
        return 0;
      }

      return count;
    } catch (e) {
      await close();
      return -1;
    }
  }
}
